from setuptools import setup

setup(
    name="lakeweed",
    version="0.1.1",
    author="TAC",
    author_email="tac@tac42.net",
    description=("Elastic converter JSON to RDB record"),
    license="MIT",
    keywords="json clickhouse",
    url="https://github.com/tac0x2a/lake_weed",
    packages=['lakeweed'],
)
